#include <QApplication>
#include <QMainWindow>
#include <QTextEdit>
#include <QClipboard>
#include <QMenu>
#include <QAction>
#include <QDebug>
#include <QToolBar>
#include <QIcon>
#include <QRandomGenerator>

class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr) : QMainWindow(parent) {
        textEdit = new QTextEdit(this);
        textEdit->setPlainText("Type some text here. Use Ctrl+C/Ctrl+V or the context menu.");
        setCentralWidget(textEdit);

        textEdit->setContextMenuPolicy(Qt::CustomContextMenu);
        connect(textEdit, &QTextEdit::customContextMenuRequested, this, &MainWindow::showContextMenu);

        // Add Toolbar
        QToolBar *toolbar = addToolBar("My Toolbar");
        addRandomIcons(toolbar, 5); // Use a constant for the number of icons.
    }

private slots:
    void showContextMenu(const QPoint &pos) {
        QMenu *menu = textEdit->createStandardContextMenu(pos);
        qDebug() << "showContextMenu qd";
        printf("showContextMenu pf");
        textEdit->append("showContextMenu\n");
/*
        // Create actions for the context menu
        QAction *copyAction = new QAction("Copy to System Clipboard", menu);
        QAction *pasteAction = new QAction("Paste from System Clipboard", menu);
        copyAction->setVisible(true);
        pasteAction->setVisible(true);

        // Connect actions to their respective slots
        connect(copyAction, &QAction::triggered, this, &MainWindow::copyToClipboard);
        connect(pasteAction, &QAction::triggered, this, &MainWindow::pasteFromClipboard);

        // Add actions to the menu
        menu->addAction(copyAction);
        menu->addAction(pasteAction);
*/
        // Show the context menu at the right position
        menu->popup(QCursor::pos()); // Use popup() to avoid blocking the UI
//        menu->exec(QCursor::pos());
    }

    void copyToClipboard() {
        QClipboard *clipboard = QApplication::clipboard();
        clipboard->setText(textEdit->textCursor().selectedText());
    }

    void pasteFromClipboard() {
        QClipboard *clipboard = QApplication::clipboard();
        if(!clipboard->text().isEmpty()){ // add check for empty clipboard.
            textEdit->textCursor().insertText(clipboard->text());
        }
    }

    void addRandomIcons(QToolBar *toolbar, int iconCount) {
        QStringList iconNames = {
            ":/qt-project.org/styles/commonstyle/images/standardbutton-open-32.png",
            ":/qt-project.org/styles/commonstyle/images/standardbutton-save-32.png",
            ":/qt-project.org/styles/commonstyle/images/standardbutton-cancel-32.png",
            ":/qt-project.org/styles/commonstyle/images/standardbutton-ok-32.png",
            ":/qt-project.org/styles/commonstyle/images/standardbutton-apply-32.png",
            ":/qt-project.org/styles/commonstyle/images/standardbutton-help-32.png"
        };
        QRandomGenerator generator;

        for (int i = 0; i < iconCount; ++i) {
            int randomIndex = generator.bounded(iconNames.size());
            QAction *action = new QAction(QIcon(iconNames.at(randomIndex)), "Random Icon", toolbar);
            toolbar->addAction(action);
        }
    }

private:
    QTextEdit *textEdit;
};

int main(int argc, char *argv[]) {
    QApplication app(argc, argv);

    MainWindow mainWindow;
    mainWindow.show();

    return app.exec();
}

#include "main.moc"